"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const d3 = require("d3");
const axes = require("./axes");
const base_1 = require("./base");
/** The base comparison data */
class CompZData extends base_1.BasePlotItem {
    /** Plot a comparison item */
    plot(group, _x, _format) {
        return this.plotSetup(group);
    }
}
exports.CompZData = CompZData;
/** A value for comparison */
class ValueZ extends CompZData {
    constructor(label, value) {
        super();
        this.lab = label;
        this.val = value;
    }
    plot(group, x, format) {
        const valueGroup = super.plot(group, x, format);
        // TODO In d3 style this should probably join on this.val, then width will
        // be a function of d, and so will the text
        valueGroup.append("g").classed(base_1.prefix + "value", true)
            .append("rect")
            .attr("y", "-0.5em")
            .attr("height", "1em")
            .attr("width", x(this.val));
        valueGroup.append("g").classed(base_1.prefix + "name", true)
            .append("text").text(this.lab);
        valueGroup.append("g").attr("transform", `translate(${x.range().reverse()[0]}, 0)`)
            .append("g").classed(base_1.prefix + "num", true)
            .append("text").text((_, i) => format(this.val, i));
        return valueGroup;
    }
}
exports.ValueZ = ValueZ;
/** A comparison plotter */
class ComparisonZ extends base_1.BasePlot {
    constructor(width) {
        super(base_1.prefix + "jeanluc-red");
        this.wdth = width;
        this.data = [];
        // Bounds
        this.maxVal = 0.0;
        this.maxSet = false;
        this.scale = d3.scaleLinear();
        this.format = d3.format("");
        // Axis
        this.label = "";
        this.ticks = [];
        this.axisFormat = d3.format("");
    }
    /** Add a value to be plotted */
    value(label, value) {
        this.maxSet || (this.maxVal = Math.max(this.maxVal, value));
        const datum = new ValueZ(label, value);
        this.data.push(datum);
        return datum;
    }
    /** Set the width of the plot */
    width(width) {
        this.wdth = width;
        return this;
    }
    /** Set the number format of the plot */
    numberFormat(format) {
        this.format = format;
        return this;
    }
    max(max) {
        if (max === undefined) {
            return this.maxVal;
        }
        else {
            this.maxVal = max;
            this.maxSet = true;
            return this;
        }
    }
    /** Set properties of the axis */
    axis({ label = this.label, ticks = this.ticks, format = this.axisFormat }) {
        this.label = label;
        this.ticks = ticks;
        this.axisFormat = format;
        return this;
    }
    name() {
        return "comparisonz";
    }
    /** Plot the current state on the svg */
    plot(svgElement) {
        const svg = super.plot(svgElement);
        const x = this.scale.range([0, this.wdth]).domain([0, this.maxVal]);
        // axes
        const axisGroup = svg
            .append("g").classed(base_1.prefix + "axis", true)
            .append("g").attr("style", `transform: translateY(${this.data.length - 1}.5em);`);
        axes.xaxis(axisGroup, x, [0, this.maxVal].concat(this.ticks), this.axisFormat, this.label);
        const dataGroup = svg.append("g");
        this.data.forEach((datum, i) => datum.plot(dataGroup, x, this.format)
            .attr("style", `transform: translateY(${i}em);`));
        return svg;
    }
}
exports.ComparisonZ = ComparisonZ;
